//
//  ContentDirectoryViewController.m
//  uPnp2
//
//  Created by rk on 10/03/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "ContentDirectoryViewController.h"
#import "DefaultAvRenderer.h"


@implementation ContentDirectoryViewController

@synthesize tableView;

- (id)init {
	self = [super initWithNibName:@"ContentDirectoryView" bundle:nil];
	
	backButtonHistory = [[NSMutableArray alloc] init];
	// These for parent UINavigationController:
	backButtonItem = [[UIBarButtonItem alloc] 
									   initWithTitle:@"Back" 
									   style:UIBarButtonItemStyleBordered 
									   target:self 
									   action:@selector(backButtonPressed:)];
	//[[self navigationItem] setLeftBarButtonItem:backButtonItem];
	[[self navigationItem] setTitle:@"Content Directory"];
	
	return self;
}

-(id)initWithServer:(UpnpAvServer*)theServer {
	self =	[self init];
	[tableView setDelegate:self];
	server = theServer;
	[server.contentDirectoryService getContainerContent:@"0" :self]; // root container has ID @"0"
	[backButtonHistory addObject:@"0"];
	return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	self = [self init];
	return self;
}


-(IBAction)backButtonPressed:(id)sender {
    
    if (currentItem) {currentItem = nil; [tableView reloadData]; return;}
	
	NSString *prevContainerId = nil;
	
	if (backButtonHistory.count > 1) {
		prevContainerId = [backButtonHistory objectAtIndex:(backButtonHistory.count - 2)];
	}
	
	if (prevContainerId) {
		[backButtonHistory removeLastObject];
		[server.contentDirectoryService getContainerContent:prevContainerId :self];
	} 
	if (backButtonHistory.count == 1) { // Only contains root container, hide the back button
		[[self navigationItem] setLeftBarButtonItem:nil];
	}
}


#pragma mark -

-(void)getContainerContentHandler:(NSArray *)theContainers: (NSArray *)theItems {
    [containers release];
    containers = [[NSMutableArray alloc] initWithCapacity:5];
	for (GDataXMLElement *containerElement in theContainers) {
		NSLog(@"%@: containerElement:%@", self, containerElement);
        UpnpAvContentDirectoryContainerModel *container = [[UpnpAvContentDirectoryContainerModel alloc] initWithGDataXMLElement:containerElement];
		[containers addObject:container];
	}
    
    [items release];
    items = [[NSMutableArray alloc] initWithCapacity:5];
	for (GDataXMLElement *itemElement in theItems) {
		NSLog(@"%@: itemElement:%@", self, itemElement);
        UpnpAvContentDirectoryItemModel *item = [[UpnpAvContentDirectoryItemModel alloc] initWithGDataXMLElement:itemElement];
		[items addObject:item];
	}
    [tableView setDelegate:self];
	[tableView setDataSource:self];
	[tableView reloadData];
}


#pragma mark -

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	[super viewDidLoad];
}



- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Overriden to allow any orientation.
	return YES;
}

#pragma mark -
#pragma mark Table view data source



- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)sectionNr {
	switch(sectionNr) {
		case CONTAINERS_SECTION:
			if (containers.count >0) {
				return [NSString stringWithFormat:@"Folders (%d)", containers.count];
			} else {
				return nil;
			}
			
			break;
		case ITEMS_SECTION:
			if (items.count >0) {
				return [NSString stringWithFormat:@"Items (%d)", items.count];
			} else {
				return nil;
			}
			
			break;
		case ITEM_DETAILS_SECTION:
			if (currentItem &&
                currentItem.resItems &&
                (currentItem.resItems.count > 0)) {
				return [NSString stringWithFormat:@"%@", currentItem.title];
			} else {
				return nil;
			}
			
			break;
	}
	return @"Other"; // default (fallthru)
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView {
	// Return the number of sections.
	return NR_SECTIONS;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	// Return the number of rows in the section.
	if ((section == CONTAINERS_SECTION) && !currentItem)
		return containers.count;
	if ((section == ITEMS_SECTION) && !currentItem)
		return items.count;
	if (section == ITEM_DETAILS_SECTION) {
        if (currentItem &&
            currentItem.resItems &&
            (currentItem.resItems.count >0))
		return currentItem.resItems.count;
    }
	
	return 0; // fallthru
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)theTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [theTableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
	}
	
	if (indexPath.section == CONTAINERS_SECTION) {
		UpnpAvContentDirectoryContainerModel *container = [containers objectAtIndex:indexPath.row];	
		cell.textLabel.text = [NSString stringWithFormat:@"%@ (%d)", container.title, container.childCount];
	}
	
	if (indexPath.section == ITEMS_SECTION) {
		UpnpAvContentDirectoryItemModel *item = [items objectAtIndex:indexPath.row];	
		cell.textLabel.text = [NSString stringWithFormat:@"%@", item.title];
	}
	
	if (indexPath.section == ITEM_DETAILS_SECTION) {
		ItemResModel *resItem = [currentItem.resItems objectAtIndex:indexPath.row];	
		cell.textLabel.text = [NSString stringWithFormat:@"%@ (%d bit) %@", 
                               resItem.resolution,
                               resItem.colorDepth,
                               resItem.url];
	}
	
	return cell;
}


/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */


/*
 // Override to support editing the table view.
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
 
 if (editingStyle == UITableViewCellEditingStyleDelete) {
 // Delete the row from the data source.
 [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
 }   
 else if (editingStyle == UITableViewCellEditingStyleInsert) {
 // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
 }   
 }
 */


/*
 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
 }
 */


/*
 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the item to be re-orderable.
 return YES;
 }
 */


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)theTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	// Navigation logic may go here. Create and push another view controller.
	/*
	 //<#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
	 // ...
	 // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController:detailViewController animated:YES];
	 [detailViewController release];
	 */
	/*
	 When a row is selected, set the detail view controller's detail item to the item associated with the selected row.
	 */
	//int sectionNr = indexPath.section;
	
	if (indexPath.section == CONTAINERS_SECTION) {
		int rowNr = indexPath.row;
		UpnpAvContentDirectoryContainerModel *container = [containers objectAtIndex:rowNr];
		NSLog(@"container:%@, title:%@", container, container.title);
		[server.contentDirectoryService getContainerContent:container.idString :self];		
		
		[backButtonHistory addObject:container.idString];
		[[self navigationItem] setLeftBarButtonItem:backButtonItem];
        currentItem = nil;
	}
	if (indexPath.section == ITEMS_SECTION) {
		int rowNr = indexPath.row;
		UpnpAvContentDirectoryItemModel *item = [items objectAtIndex:rowNr];
		NSLog(@"item:%@, item.title:%@, item.resItems.count:%d", 
              item, 
              item.title,
              item.resItems.count
              );
        currentItem = item;
        [tableView reloadData];
	}
	if (indexPath.section == ITEM_DETAILS_SECTION) { 
		int rowNr = indexPath.row;
		ItemResModel *resItem = [currentItem.resItems objectAtIndex:rowNr];
		NSLog(@"resItem:%@, %@ resItem.url:%@ resItem.protocolInfo:%@", 
              resItem,
              resItem.resolution,
              resItem.url,
              resItem.protocolInfo
              );
        [server.contentDirectoryService fetchResource:resItem.url];  //TODO This is broken
        UpnpAvRenderer *defaultRenderer = [DefaultAvRenderer singletonInstance];
        UpnpAvTransportService *transportService =[defaultRenderer transportService];
        [transportService asynchSetAvTransportUri:resItem.url: nil];
        //currentItem = item;
	}
	
}

#pragma mark -


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc. that aren't in use.
}


- (void)viewDidUnload {
	[super viewDidUnload];
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[super dealloc];
}


@end
